package kw4h;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

/**
 * @author Kw4h
 *
 */
public class ConvHull 
{
	// all solarsystems
	private Hashtable allSystems;
	private Vector points = new Vector();
	private Vector sortedPoints = new Vector();
	private Vector resultPoints = new Vector();
	
	/**
	 * 
	 * @param solarSystems
	 */
	public ConvHull(Hashtable solarSystems)
	{
		this.allSystems = solarSystems;
	}
	
	/**
	 * 
	 * @param region
	 */
	public Vector calculate(Region region)
	{		
		// clear all points
		this.points.clear();
		this.resultPoints.clear();
		this.sortedPoints.clear();
		
		// loop through all solarsystems - and create a convexpoint if in the region
		Enumeration num = this.allSystems.elements();
		while (num.hasMoreElements())
		{
			SolarSystem current = (SolarSystem) num.nextElement();
			
			// if both have the same regionid
			if (current.regionID == region.id)
			{
				// create point
				ConvexPoint point = new ConvexPoint(current.x, current.y, current.z);
				
				// add point to the points
				points.add(point);
			}
		}
		
		// we have all the points now - calculate the extreme point
		this.setMinMaxZ(region);
		ConvexPoint extreme = this.findExtreme();
		
		// we have the extreme point - remove it from the points stack and add it to the result points
		this.points.remove(extreme);
		this.resultPoints.add(extreme);
		
		// now calculate all the angles, and sort them
		this.calcAngles();
		this.sortByAngle();
		this.runThrough();
		
		return this.resultPoints;
	}
	
	/**
	 * 
	 * @param r
	 */
	private void setMinMaxZ(Region r)
	{
		// return point object - starts at null
		double minZ = 999999999;
		double maxZ = -999999999;
		
		// find the lowest y coordinate
		Enumeration num = this.points.elements();
		while (num.hasMoreElements())
		{
			ConvexPoint current = (ConvexPoint) num.nextElement();
			
			// if extreme is null, make it this one
			if (current.y > maxZ)
			{
				maxZ = current.y;
			}
			
			if (current.y < minZ)
			{
				minZ = current.y;
			}
			
		}
		
		// we found the min-max, set it
		r.setMinMax(minZ, maxZ);
	}
	
	/**
	 * 
	 * @return
	 */
	private ConvexPoint findExtreme()
	{
		// return point object - starts at null
		ConvexPoint extreme = null;
		
		// find the lowest y coordinate
		Enumeration num = this.points.elements();
		while (num.hasMoreElements())
		{
			// if extreme is null, make it this one
			if (extreme == null)
			{
				extreme = (ConvexPoint) num.nextElement();
			}
			else
			{
				ConvexPoint current = (ConvexPoint) num.nextElement();
				
				// if the y is lower than the now set extreme
				if (current.z < extreme.z)
				{
					extreme = current;
				}
			}
		}
		
		// return the extreme
		return extreme;
	}
	
	/**
	 * 
	 *
	 */
	private void calcAngles()
	{
		// calculate the angles here
		// get the extreme point
		ConvexPoint extreme = (ConvexPoint) this.resultPoints.elements().nextElement();
		
		// then loop through all the other points
		Enumeration num = this.points.elements();
		while(num.hasMoreElements())
		{
			// get the current element
			ConvexPoint current = (ConvexPoint) num.nextElement();
			
			// vars for pythagoras
			double width, height, angle;
			width = current.x - extreme.x;
			height = current.z - extreme.z;
			
			// now we know all sides of the triangle needed for angle calculation
			// tan ( width / height) - note: Math.toDegrees = key
			angle = Math.toDegrees(Math.atan(width / height));
			current.setAngle(angle);
		}
	}
	
	/**
	 * 
	 *
	 */
	private void sortByAngle()
	{
		// very intense procedure. While the list is not empty,
		// find the largest angle, and add it to the sorted list (and remove from orig list)
		while(!this.points.isEmpty())
		{
			ConvexPoint largest = null;
			
			// loop through them all
			Enumeration num = this.points.elements();
			while (num.hasMoreElements())
			{
				// if the largest is empty, fill it
				ConvexPoint current = (ConvexPoint) num.nextElement();
				if (largest == null)
				{
					largest = current;
				}
				else
				{
					// check if the angle is larger than the current largest
					if (current.angle >= largest.angle)
					{
						largest = current;
					}
				}
			}
			
			// we found the largest - remove from points, add to sortedpoints
			this.points.remove(largest);
			this.sortedPoints.add(largest);
		}
	}

	/**
	 * 
	 *
	 */
	private void runThrough()
	{
		// top = last point added to stack
		// second = the one added before that
		// the extreme point already exists, now add the first in the sorted points
		ConvexPoint second = (ConvexPoint) this.sortedPoints.firstElement();
		ConvexPoint top = (ConvexPoint) this.resultPoints.firstElement();
		
		this.resultPoints.add(second);
		this.sortedPoints.remove(second);
		
		top = (ConvexPoint) this.resultPoints.elementAt(1);
		second = (ConvexPoint) this.resultPoints.elementAt(0);
		
		for(Iterator i = this.sortedPoints.iterator(); i.hasNext();)
		{
			ConvexPoint current = (ConvexPoint) i.next();
			Double cProduct = this.crossProduct(second, top, current);
			
			// straight line
			if (cProduct == 0)
			{
				// remove last resultpoint
				this.resultPoints.remove(this.resultPoints.lastElement());
				
				// voeg deze toe
				this.resultPoints.add(current);
				
				// new second en top
				// current = new top
				// second is previous top
				second = top;
				top = current;
			}
			// good corner to the right
			else if (cProduct > 0)
			{
				this.resultPoints.add(current);
				
				// new second en top
				// current = new top
				// second is previous top
				second = top;
				top = current;
				
			}
			// wrong corner
			else
			{
				// this is the trackback part
				while (cProduct <= 0 && this.resultPoints.size() > 2)
				{
					// remove the last point
					int iSecond = this.resultPoints.indexOf(second);
					this.resultPoints.remove(top);
					
					// new top and second
					top = second;
					second = (ConvexPoint) this.resultPoints.elementAt(iSecond - 1);
					
					// calculate new crossProduct
					cProduct = this.crossProduct(second, top, current);
				}
				
				// add the current afterwards
				this.resultPoints.add(current);
				second = top;
				top = current;
			}
			
		}
	}
	
	/**
	 * 
	 * @param p1
	 * @param p2
	 * @param p3
	 * @return
	 */
	private double crossProduct(ConvexPoint p1, ConvexPoint p2, ConvexPoint p3)
	{
		double result = ((p2.x - p1.x) * (p3.z - p1.z)) - ((p3.x - p1.x) * (p2.z - p1.z));
		return result;
	}
}
