package kw4h;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;
import java.util.Vector;

import kw4h.SolarSystem;
import kw4h.Region;

/**
 * @author Kw4h
 *
 */
public class DBReader 
{
	private Connection conn = null;
	private Statement statement;
	
	// drawscale
	private double drawScale = 4.8445284569785E14;
	
	/**
	 * Constructor
	 */
	public DBReader()
	{		
		// new database
		try {
			
			Class.forName("org.hsqldb.jdbcDriver");
			this.conn = DriverManager.getConnection("jdbc:hsqldb:file:./database/db");
			this.statement = this.conn.createStatement();
			
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 *
	 */
	public Hashtable getSolarSystems()
	{
		// create the query - TOP 5
		String query = "SELECT x, y, z, solarSystemID, constellationID, regionID, solarSystemName, security FROM mapSolarSystems;";
		Hashtable systems = new Hashtable();
		
		try
		{
			// execute the query
			ResultSet result = this.statement.executeQuery(query);
			
			// loop through results
			while (result.next())
			{
				// new SolarSystem
				int ssID, clID, rgID;
				double x, y, z, security;
				String name;
				x = result.getDouble(1) / this.drawScale;
				y = result.getDouble(2) / this.drawScale;
				z = (-result.getDouble(3)) / this.drawScale;
				ssID = result.getInt(4);
				clID = result.getInt(5);
				rgID = result.getInt(6);
				name = result.getString(7);
				security = result.getDouble(8);
				
				// create new solarsystem object, and add it to the list
				SolarSystem sys = new SolarSystem(ssID, x, y, z, clID, rgID, name, security);
				systems.put(ssID, sys);
			}
			
			result.close();
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}
		
		return systems;
	}
	
	/**
	 * 
	 *
	 */
	public Hashtable getSystemJumps(Hashtable solarSystems)
	{
		String query = "SELECT fromSolarSystemID, toSolarSystemID FROM mapSolarSystemJumps;";
		Hashtable jumps = new Hashtable();
		
		try 
		{
			// execute query
			ResultSet result = this.statement.executeQuery(query);
			
			// loop through results
			while (result.next())
			{
				// de from and to solarsystem
				SolarSystem from = (SolarSystem) solarSystems.get(result.getInt(1));
				SolarSystem to = (SolarSystem) solarSystems.get(result.getInt(2));
				
				// they both must exist
				if (from != null && to != null)
				{
					// retrieve an existing from
					Vector v = (Vector) jumps.get(from);
					
					// from doesn't exist yet, so create one
					if (v == null)
					{
						// add new vector for destinations
						v = new Vector();
						jumps.put(from, v);
					}
					// add the destination to the from
					v.addElement(to);
				}
			}
			
			result.close();
		}
		catch (SQLException e) 
		{
			e.printStackTrace();
		}
		
		return jumps;
	}
	
	/**
	 * 
	 *
	 */
	public Hashtable getRegions()
	{
		// create the query
		String query = "SELECT regionID, regionName, x, y, z FROM mapRegions;";
		Hashtable regions = new Hashtable();
		
		try 
		{
			// execute query
			ResultSet result = this.statement.executeQuery(query);
			
			// loop through results
			while(result.next())
			{
				int id = result.getInt(1);
				String name = result.getString(2);
				double x = result.getDouble(3) / this.drawScale;
				double y = result.getDouble(4) / this.drawScale;
				double z = (-result.getDouble(5)) / this.drawScale;
				
				Region region = new Region(id, name, x, y, z);
				regions.put(id, region);
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}
		
		return regions;
	}
	
	/**
	 * 
	 * @param part
	 * @return
	 */
	public Vector findSolarSystem(String part)
	{
		Vector result = new Vector();
		String query = "SELECT solarSystemName FROM mapSolarSystems WHERE LOWER(solarSystemName) LIKE '" + part.toLowerCase() + "%';";
		
		try 
		{
			// add found solarSystems
			ResultSet rs = this.statement.executeQuery(query);
			while (rs.next())
			{
				String sys = rs.getString(1);
				result.add(sys);
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}
		
		return result;
	}
	
	/**
	 * 
	 * @param sys
	 * @return
	 */
	public Boolean systemExists(String sys)
	{
		String query = "SELECT DISTINCT solarSystemName FROM mapSolarSystems WHERE solarSystemName = '" + sys + "';";
		
		if(sys != null && sys != "")
		{
			try 
			{
				ResultSet rs = this.statement.executeQuery(query);
				if (rs.next())
				{
					return true;
				}
				else
				{
					return false;
				}
			} 
			catch (SQLException e) 
			{
				e.printStackTrace();
			}
		}
			
		return false;
	}
}
