package kw4h;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;
import java.util.Vector;

import javax.media.opengl.GLCanvas;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import com.sun.opengl.util.Animator;

/**
 * 
 * @author Kw4h
 *
 */
public class Evemap
{
	// display objects
	public GLCanvas canvas;
	public JFrame frame;
	private Renderer renderer;
	
	// panel for the left menu
	private JPanel leftPanel;
	
	// subpanel for view settings
	private JPanel viewPanel;
	private JButton flatButton, outlineButton, systemButton, tacticalButton, secstatButton;
	
	// subpanel for planning route
	private JPanel routePanel;
	private JLabel wpLabel;
	private JComboBox wpBox;
	private JList wpList;
	private JButton wpAddButton, wpRemoveButton;
	private ButtonGroup avoidGroup;
	private JRadioButton avoidLowRadio, avoidHighRadio, avoidNoRadio;
	
	// subpanel for system search
	private JPanel searchPanel;
	private JComboBox searchBox;
	private JButton searchButton;
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) 
	{
		new Evemap();
	}
	
	/**
	 * 
	 *
	 */
	public Evemap()
	{
		// set system look and feel
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e1) {
		} catch (InstantiationException e1) {
		} catch (IllegalAccessException e1) {
		} catch (UnsupportedLookAndFeelException e1) {
		}
		
		// create a frame
		this.frame = new JFrame("EvE Map - v0.3");
		
		// make a nice window size
		this.frame.setSize(1024, 768);
		this.frame.setBackground(Color.black);
		this.frame.setLocation(100, 100);
		this.frame.setLayout(new BorderLayout());
		
		this.setupInterface(this.frame.getContentPane());
		this.setupCanvas(this.frame.getContentPane());
		this.frame.setVisible(true);
		
		// creating an animator
		final Animator animator = new Animator(canvas);
		
		// of course, a window adapter
		this.frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				// its been suggested to run this on a different thread
				// otherwise it could be conflicting (animator won't stop before close)
				new Thread(new Runnable()
					{
						public void run()
						{
							animator.stop();
							System.exit(0);
						}
					}				
				).start();
			}
		});
		
		// sleep for a lil bit to draw the interface
		try {
			Thread.sleep(100);
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		animator.setRunAsFastAsPossible(true);
		animator.start();

	}
	
	/**
	 * 
	 * @param container
	 */
	private void setupInterface(Container container)
	{
		// main left panel
		this.leftPanel = new JPanel();
		this.leftPanel.setPreferredSize(new Dimension(180, container.getHeight()));
		container.add(this.leftPanel, BorderLayout.LINE_START);
		
		// sub panel for view settings
		this.viewPanel = new JPanel();
		this.viewPanel.setPreferredSize(new Dimension(170, 160));
		this.viewPanel.setBorder(BorderFactory.createTitledBorder("View"));
		this.leftPanel.add(this.viewPanel);
		
		// add buttons for view settings
		this.flatButton = new JButton("Flatten map");
		this.flatButton.setPreferredSize(new Dimension(110, 20));
		this.flatButton.addActionListener(new BtnListener("flatten"));
		
		this.outlineButton = new JButton("Toggle outlines");
		this.outlineButton.setPreferredSize(new Dimension(110, 20));
		this.outlineButton.addActionListener(new BtnListener("outline"));
		
		this.systemButton = new JButton("System names");
		this.systemButton.setPreferredSize(new Dimension(110, 20));
		this.systemButton.addActionListener(new BtnListener("system"));
		
		this.tacticalButton = new JButton("Tactical view");
		this.tacticalButton.setPreferredSize(new Dimension(110, 20));
		this.tacticalButton.addActionListener(new BtnListener("tactical"));
		this.tacticalButton.setEnabled(false);
		
		this.secstatButton = new JButton("Security status");
		this.secstatButton.setPreferredSize(new Dimension(110, 20));
		this.secstatButton.addActionListener(new BtnListener("showsecstat"));
		
		this.viewPanel.add(this.flatButton);
		this.viewPanel.add(this.outlineButton);
		this.viewPanel.add(this.systemButton);
		this.viewPanel.add(this.tacticalButton);
		this.viewPanel.add(this.secstatButton);
		
		
		// sub panel for route planning
		this.routePanel = new JPanel();
		this.routePanel.setPreferredSize(new Dimension(170, 250));
		this.routePanel.setBorder(BorderFactory.createTitledBorder("Plan route"));
		this.leftPanel.add(this.routePanel);
		
		// add stuff for route planning
		this.wpLabel = new JLabel("Waypoint:");
		this.wpLabel.setPreferredSize(new Dimension(150, 18));
		
		this.wpBox = new JComboBox();
		this.wpBox.setPreferredSize(new Dimension(80, 20));
		this.wpBox.setEditable(true);
		this.wpBox.addActionListener(new BtnListener("combobox"));
		
		this.wpList = new JList();
		this.wpList.setPreferredSize(new Dimension(80, 100));
		this.wpList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.wpList.setBorder(BorderFactory.createEtchedBorder(1));
		
		this.avoidHighRadio = new JRadioButton("Prefer lowsec");
		this.avoidHighRadio.setPreferredSize(new Dimension(150, 16));
		this.avoidHighRadio.addActionListener(new BtnListener("radiobtn"));
		
		this.avoidLowRadio = new JRadioButton("Prefer highsec");
		this.avoidLowRadio.setPreferredSize(new Dimension(150, 16));
		this.avoidLowRadio.addActionListener(new BtnListener("radiobtn"));
		
		this.avoidNoRadio = new JRadioButton("Don't avoid");
		this.avoidNoRadio.setPreferredSize(new Dimension(150, 16));
		this.avoidNoRadio.setSelected(true);
		this.avoidNoRadio.addActionListener(new BtnListener("radiobtn"));
		
		this.wpAddButton = new JButton("Add");
		this.wpAddButton.setPreferredSize(new Dimension (60, 20));
		this.wpAddButton.addActionListener(new BtnListener("addwp"));
		
		this.wpRemoveButton = new JButton("Del");
		//this.wpRemoveButton.setEnabled(false);
		this.wpRemoveButton.setPreferredSize(new Dimension (60, 20));
		this.wpRemoveButton.addActionListener(new BtnListener("remwp"));
		
		this.avoidGroup = new ButtonGroup();
		this.avoidGroup.add(this.avoidNoRadio);
		this.avoidGroup.add(this.avoidLowRadio);
		this.avoidGroup.add(this.avoidHighRadio);
		
		this.routePanel.add(this.wpLabel);
		this.routePanel.add(this.wpBox);
		this.routePanel.add(this.wpAddButton);
		this.routePanel.add(this.wpList);
		this.routePanel.add(this.wpRemoveButton);
		this.routePanel.add(this.avoidNoRadio);
		this.routePanel.add(this.avoidLowRadio);
		this.routePanel.add(this.avoidHighRadio);
		
		
		// subpanel for system search
		this.searchPanel = new JPanel();
		this.searchPanel.setPreferredSize(new Dimension(170, 60));
		this.searchPanel.setBorder(BorderFactory.createTitledBorder("Find a system"));
		this.leftPanel.add(this.searchPanel);
		
		this.searchBox = new JComboBox();
		this.searchBox.setPreferredSize(new Dimension(80, 20));
		this.searchBox.setEditable(true);
		this.searchBox.addActionListener(new BtnListener("combobox"));
		
		this.searchButton = new JButton("Find");
		this.searchButton.setPreferredSize(new Dimension (60, 20));
		this.searchButton.addActionListener(new BtnListener("searchsystem"));
		
		this.searchPanel.add(this.searchBox);
		this.searchPanel.add(this.searchButton);
	}
	
	/**
	 * 
	 * @param container
	 */
	private void setupCanvas(Container container)
	{
		// Create opengl canvas, and add it to the frame
		this.canvas = new GLCanvas();
		container.add(canvas);
		this.renderer = new Renderer(this.frame.getWidth(), this.frame.getHeight());
		this.canvas.addGLEventListener(this.renderer);
		this.canvas.addMouseListener(this.renderer);
		this.canvas.addMouseMotionListener(this.renderer);
		this.canvas.addKeyListener(this.renderer);
		this.canvas.addMouseWheelListener(this.renderer);
	}
	
	/**
	 * 
	 * @author Kw4h
	 *
	 */
	private class BtnListener implements ActionListener
	{
		private String action;
		
		/**
		 * 
		 * @param action
		 */
		public BtnListener(String action)
		{
			this.action = action;
		}
		
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e) 
		{
			// do stuff based on action
			// flatten map
			if (this.action.equals("flatten"))
			{
				renderer.toggleFlat();
			}
			// outline
			else if (this.action.equals("outline"))
			{
				renderer.toggleOutline();
			}
			// draw system names
			else if (this.action.equals("system"))
			{
				renderer.toggleSystem();
			}
			// toggle tactical
			else if (this.action.equals("tactical"))
			{
				renderer.toggleTactical();
			}
			// pressing enter on combobox
			else if (this.action.equals("combobox"))
			{
				if (e.getActionCommand().equals("comboBoxEdited"))
				{
					// get what is put in
					JComboBox source = (JComboBox) e.getSource();
					String input = (String) source.getSelectedItem();
					source.removeAllItems();
					
					// only two letters or more
					if (input.length() >= 2)
					{
						Vector found = renderer.getDBReader().findSolarSystem(input);
						
						if (found.size() > 1)
						{
							source.addItem(input);
						}
						
						for (Iterator i = found.iterator(); i.hasNext();)
						{
							source.addItem(i.next());
						}
						
						// show popup
						if (found.size() > 1)
						{
							source.showPopup();
						}
						else if (found.size() == 1)
						{
							renderer.travelSystems.add(found.firstElement());
							
							// update the list
							wpList.setListData(renderer.travelSystems);
							
							int avoidance = 0;
							if (avoidLowRadio.isSelected())
							{
								avoidance = 1;
							}
							else if (avoidHighRadio.isSelected())
							{
								avoidance = 2;
							}
							
							// start calculation
							renderer.calcWaypoints(avoidance);
						}
					}
					else
					{
						source.addItem(input);
					}
				}
			}
			// hit the search system button
			else if (this.action.equals("searchsystem"))
			{
				String sys = (String) searchBox.getSelectedItem();
				renderer.findSystem(sys);
				
				searchBox.removeAllItems();
			}
			// hit the add wp button
			else if (this.action.equals("addwp"))
			{
				// get the system name and add to the list
				String sys = (String) wpBox.getSelectedItem();
				if (sys != null && renderer.getDBReader().systemExists(sys) && !renderer.travelSystems.contains(sys))
				{
					renderer.travelSystems.add(sys);
					
					// update the list
					wpList.setListData(renderer.travelSystems);
					
					int avoidance = 0;
					if (avoidLowRadio.isSelected())
					{
						avoidance = 1;
					}
					else if (avoidHighRadio.isSelected())
					{
						avoidance = 2;
					}
					
					// start calculation
					renderer.calcWaypoints(avoidance);
				}
				
				// clear input box
				wpBox.removeAllItems();
			}
			else if (this.action.equals("remwp"))
			{
				// remove item from list, and update the view
				renderer.travelSystems.remove(wpList.getSelectedValue());
				wpList.setListData(renderer.travelSystems);
				
				// recalculate
				int avoidance = 0;
				if (avoidLowRadio.isSelected())
				{
					avoidance = 1;
				}
				else if (avoidHighRadio.isSelected())
				{
					avoidance = 2;
				}
				
				renderer.calcWaypoints(avoidance);
			}
			else if (this.action.equals("showsecstat"))
			{
				renderer.toggleSecStat();
			}
			else if (this.action.equals("radiobtn"))
			{
				// recalculate
				int avoidance = 0;
				if (avoidLowRadio.isSelected())
				{
					avoidance = 1;
				}
				else if (avoidHighRadio.isSelected())
				{
					avoidance = 2;
				}
				
				renderer.calcWaypoints(avoidance);
			}
		}
	}
}
