package kw4h;
// TODO: constellation names on distance
// TODO: calculate cyno jumps
// TODO: system search - partial names
// TODO: route planning with partial names
 
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;
import com.sun.opengl.util.GLUT;

import kw4h.DBReader;

/**
 * 
 * @author Kw4h
 *
 */
public class Renderer implements GLEventListener, MouseListener, MouseMotionListener, KeyListener, MouseWheelListener
{	
	private GLU glu;
	private GLUT glut;
	
	// certain vars
	private static final double SYSRANGE = 200;
	private static final double REGIONRANGE = 450;
	
	// retrieving data objects
	private DBReader dbReader;
	private Hashtable solarSystems = new Hashtable();
	private Hashtable systemJumps = new Hashtable();
	private Hashtable regions = new Hashtable();
	private Hashtable convexSystems = new Hashtable();
	
	// travel jumps
	private Vector travelJumps = new Vector();
	public Vector travelSystems;
	private int avoidance = 0;
	
	// search system
	private String searchedSystem = null;
	
	// heightcale
	private double heightScale = 1;
	private boolean drawFlat = false;
	
	// mode vars
	private boolean tactical = false;
	private boolean drawSystemNames = false;
	private boolean isReady = false;
	private boolean drawOutlines = false;
	private boolean showSecStat = false;
	
	// draw offsets
	private double offsetX, offsetZ;
	long lastFrameTime; // always negative, for easy calculations
	
	// mouse vars
	private int mStartX, mStartY;
	private int mMoveX, mMoveY, mMoveZ;
	private int btnsPressed = 0;
	
	// view vars
	private double viewX, viewY, viewZ;
	
	/**
	 * Constructor
	 *
	 */
	public Renderer(int width, int height)
	{		
		this.offsetX = (double) (width);
		this.offsetZ = (double) (height);
		
		this.glu = new GLU();
		this.glut = new GLUT();
	}
	
	/**
	 * Initialising of the opengl world
	 */
	public void init(GLAutoDrawable drawable) {		
	}
	
	/**
	 * 
	 */
	public void display(GLAutoDrawable drawable) 
	{
		if (this.isReady == false)
		{
			GL gl = drawable.getGL();			
			this.doInit(gl);
		}
		
		if (this.isReady == true)
		{
			// get the GL object
			GL gl = drawable.getGL();
			
			// clear the buffer
			gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
			
			// setup the view
			this.setupView(gl);
			
			// set the modelview
			gl.glMatrixMode(GL.GL_MODELVIEW);
			gl.glLoadIdentity();
			
			// set the heightScale
			this.setHeightScale();
			
			this.drawSystemJumps(gl);
			this.drawSolarSystems(gl);
			this.drawRegions(gl);
			this.drawTravelPath(gl);
			
			// 2d stuff
			this.draw2d(gl);
			
			gl.glFlush();
		}
	}

	public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {
	}

	/**
	 * Resizing of viewport
	 */
	public void reshape(GLAutoDrawable drawable, int x, int y, int width,int height) 
	{
		// ge the GL object
		GL gl = drawable.getGL();
		
		// set up the projection (lens for camera)
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		this.glu.gluPerspective(60.0f, (double) width/height, 5.0, 800000.0);
		//this.glu.gluLookAt(0.0, 0.0, 500.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
		gl.glTranslated(this.viewX, this.viewY, this.viewZ);
		gl.glViewport(0, 0, width, height);
		
		// offsets
		this.offsetX = (double) (width);
		this.offsetZ = (double) (height);
		
	}

	/* ==============================================*
	 * Custom methods start here, like drawing a line*
	 * ==============================================*/
	
	/**
	 * 
	 */
	private void doInit(GL gl)
	{		
		this.dbReader = new DBReader();
		this.solarSystems = this.dbReader.getSolarSystems();
		
		// Sleep - so it has time to process
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		this.systemJumps = this.dbReader.getSystemJumps(this.solarSystems);
		
		// Sleep - so it has time to process
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		this.regions = this.dbReader.getRegions();
		this.travelSystems = new Vector();
		
		ConvHull hullCalc = new ConvHull(this.solarSystems);
		
		Enumeration num = this.regions.elements();
		while (num.hasMoreElements())
		{
			Region r = (Region) num.nextElement();
			Vector rResult = hullCalc.calculate(r);
			
			this.convexSystems.put(r, rResult.clone());
		}
		
		// default mouse vars
		this.mStartX = -1;
		this.mStartY = -1;
		this.mMoveX = 1;
		this.mMoveY = 0;
		this.mMoveZ = 0;
		
		// default view location
		this.viewX = 1;
		this.viewY = 0;
		this.viewZ = -500;
		
		// set bg to black - and rendering to flat
		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		gl.glClearDepth(1.0f);
		
		// sexy lines - enabling smooth lines and funky blending at the nicest quality
		gl.glEnable(GL.GL_LINE_SMOOTH);
		gl.glEnable(GL.GL_BLEND);
		gl.glBlendFunc( GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
		gl.glShadeModel(GL.GL_SMOOTH);
		gl.glHint(GL.GL_LINE_SMOOTH_HINT, GL.GL_NICEST);
		
		gl.glEnable(GL.GL_DEPTH_TEST);
		gl.glDepthFunc(GL.GL_LEQUAL);
		
		this.glu.gluPerspective(60.0f, this.offsetX / this.offsetZ, 1.0f, 500.0f);
		
		this.isReady = true;
	}
	
	/**
	 * 
	 */
	private void setupView(GL gl)
	{
		// if i drag the screen
		if (this.mMoveX != 0 || this.mMoveY != 0 || this.mMoveZ != 0)
		{
			// load projection matrix
			gl.glMatrixMode(GL.GL_PROJECTION);
			gl.glLoadIdentity();
			this.glu.gluPerspective(60.0f, this.offsetX / this.offsetZ, 5.0, 800000.0);
			gl.glViewport(0, 0, (int) this.offsetX, (int) this.offsetZ);
			
			// set new x, y and z for the cam
			double accel = ((double) this.viewZ / 825);
			if (accel >= -0.2)
			{
				accel = -0.2;
			}
			
			this.viewX += (this.mMoveX * -accel);
			this.viewY += (this.mMoveY * accel);
			this.viewZ += (this.mMoveZ * -accel);
			
			// we're not going any lower than -15
			if (this.viewZ > 0)
			{
				this.viewZ = 0;
			}
			
			// set the move to 0
			this.mMoveX = 0;
			this.mMoveY = 0;
			this.mMoveZ = 0;
			
			// transform
			gl.glTranslated(this.viewX, this.viewY, this.viewZ);
		}
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void draw2d(GL gl)
	{
		// set up the ortho view for 2d drawing
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glPushMatrix();
		gl.glLoadIdentity();
		gl.glOrtho(0, this.offsetX, this.offsetZ, 0, -1, 1);
		gl.glMatrixMode(GL.GL_MODELVIEW);
		
		// do the actual drawing here
		// draw jumps, if any
		this.drawJumpsMenu(gl);
		
		// draw shortcut menu
		this.drawShorts(gl);
		
		// draw copyright
		this.drawCopyright(gl);
		
		// set the view back
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glPopMatrix();
		gl.glMatrixMode(GL.GL_MODELVIEW);
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawJumpsMenu(GL gl)
	{
		// only draw if there's any jumps
		if (this.travelJumps.size() > 0)
		{
			// set the color
			gl.glColor3f(0.9f, 0.9f, 0.9f);
			
			// begin drawing the outlines
			gl.glPointSize(1.0f);
			gl.glBegin(GL.GL_LINES);
			
			// horizontal line
			gl.glVertex2f(10, 10);
			gl.glVertex2f(130, 10);
			gl.glEnd();
			
			// loop through all jumps
			int num = 1;
			for (Iterator y = this.travelJumps.iterator(); y.hasNext();)
			{
				Vector v = (Vector) y.next();
				for (Iterator i = v.iterator(); i.hasNext();)
				{
					SolarSystem sys = (SolarSystem) i.next();
					
					// get security status
					double security = sys.security;
					if (security < 0) security = 0.0;
					
					String sec = String.valueOf(security).substring(0, 3);
					
					// draw the solarsystem name - and sec status
					gl.glRasterPos2d(15, (16 * num) + 10);
					this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, sys.name);
					
					gl.glRasterPos2d(100, (16 * num) + 10);
					this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, "(" + sec + ")");
					num ++;
				}
			}
			
			// vertical line
			gl.glBegin(GL.GL_LINES);
			gl.glVertex2f(10, 10);
			gl.glVertex2f(10, (num * 16) + 10);
			gl.glEnd();
		}
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawShorts(GL gl)
	{
		// draw menu border
		gl.glColor3f(0.9f, 0.9f, 0.9f);
		gl.glBegin(GL.GL_LINES);
		
		// horizontal line
		gl.glVertex2f((float)this.offsetX - 10, 10);
		gl.glVertex2f((float)this.offsetX - 170, 10);
		
		gl.glEnd();
		
		// start drawing text
		int i = 1;
		
		gl.glRasterPos2d(this.offsetX - 170, (16 * i) + 10);
		this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, "(f) Flatten map: " + String.valueOf(this.drawFlat));
		i++;
		
		gl.glRasterPos2d(this.offsetX - 170, (16 * i) + 10);
		this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, "(o) Draw outlines: " + String.valueOf(this.drawOutlines));
		i++;
		
		gl.glRasterPos2d(this.offsetX - 170, (16 * i) + 10);
		this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, "(s) Security status: " + String.valueOf(this.showSecStat));
		i++;
		
		gl.glRasterPos2d(this.offsetX - 170, (16 * i) + 10);
		this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, "(n) Solarsystem names: " + String.valueOf(this.drawSystemNames));
		i++;
		
		/*gl.glRasterPos2d(this.offsetX - 170, (16 * i) + 10);
		this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, "(t) Tactical view: " + String.valueOf(this.tactical));
		i++;*/
		
		// vertical line
		gl.glBegin(GL.GL_LINES);
		gl.glVertex2f((float)this.offsetX - 10, 10);
		gl.glVertex2f((float)this.offsetX - 10, (16 * (i) + 10));
		gl.glEnd();
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawCopyright(GL gl)
	{
		gl.glColor4f(1, 1, 1, 0.2f);
		gl.glRasterPos2d(this.offsetX - 70, this.offsetZ - 5);
		this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, "2007 Kw4h");
	}
	
	/**
	 * 
	 */
	private void drawLine(GL gl, double startx, double starty, double startz, double endx, double endy, double endz, float R, float G, float B)
	{
		// set the color
		gl.glColor3f(R, G, B);
		
		// begin the draw session
		gl.glBegin(GL.GL_LINES);
		
		// normal is pointing up
		gl.glVertex3d((startx), (startz), (starty * this.heightScale));
		
		gl.glVertex3d((endx), (endz), (endy * this.heightScale));
		
		// end the draw session
		gl.glEnd();
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawSolarSystems(GL gl)
	{	
		// system dots always on top
		gl.glDisable(GL.GL_DEPTH_TEST);
		
		// loop through solarsystems
		Enumeration num = this.solarSystems.elements();
		while(num.hasMoreElements())
		{
			// get the solarsystem element
			SolarSystem sys = (SolarSystem) num.nextElement();
			
			// draw the found system
			if (sys.name.equalsIgnoreCase(this.searchedSystem))
			{
				this.drawFoundSystem(gl, sys);
			}
			
			// set the color and size
			gl.glColor3f(0.4f, 0.4f, 0.6f);
			gl.glPointSize(2.0f);
			
			// get security status
			if (this.showSecStat)
			{
				double security = sys.security;
				if (security < 0) security = 0.0;
				double sec = Double.parseDouble(String.valueOf(security).substring(0, 3));
				
				float r, g, b;
				r = 0.4f;
				g = 0.4f;
				b = 0.6f;
				
				// very low sec
				if (sec <= 0.2)
				{
					r = 0.7f;
					g = 0.0f;
					b = 0.0f;
				}
				// low sec
				else if (sec <= 0.4)
				{
					r = 0.7f;
					g = 0.4f;
					b = 0.0f;
				}
				// med sec
				else if (sec <= 0.6)
				{
					r = 1.0f;
					g = 0.6f;
					b = 0.0f;
				}
				// med sec
				else if (sec <= 0.8)
				{
					r = 0.9f;
					g = 0.9f;
					b = 0.0f;
				}
				// high sec
				else
				{
					r = 0.6f;
					g = 1.0f;
					b = 0.0f;
				}
				
				gl.glPointSize(3f);
				gl.glColor3f(r, g, b);
			}
			
			// start drawing
			gl.glBegin(GL.GL_POINTS);
			
			// add a single point - normal points upwards
			gl.glVertex3d((sys.x), (sys.z), (sys.y * this.heightScale));
			
			// end drawing
			gl.glEnd();
			
			// check if the distance is lower than sysrange
			if (this.drawSystemNames)
			{
				if (KwCalc.calcDistance(-sys.x, -sys.z, (sys.y * this.heightScale), this.viewX, this.viewY, -this.viewZ) < Renderer.SYSRANGE)
				{
					// draw the text
					gl.glColor3f(0.5f, 0.5f, 0.5f);
					gl.glRasterPos3d((sys.x) + 0.25f, (sys.z), (sys.y * this.heightScale));
					this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, sys.name);
				}
			}
		}
		gl.glEnable(GL.GL_DEPTH_TEST);
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawSystemJumps(GL gl)
	{
		// systemJumps: SolarSystem,vector
		// vector has all destinations
		Enumeration num = this.systemJumps.keys();
		
		// loop through all the from's
		while(num.hasMoreElements())
		{
			SolarSystem from = (SolarSystem) num.nextElement();
			Vector toSystems = (Vector) this.systemJumps.get(from);
			
			// loop to all destinations
			Iterator it = toSystems.iterator();
			while(it.hasNext())
			{
				SolarSystem to = (SolarSystem) it.next();
				
				// linecolor
				float r, g, b;
				
				// region jump
				if (from.regionID != to.regionID)
				{
					r = 0.5f;
					g = 0.0f;
					b = 0.5f;
				}
				// constellationjump
				else if (from.constellationID != to.constellationID)
				{
					r = 0.5f;
					g = 0.0f;
					b = 0.0f;
				}
				// normal jump
				else
				{
					r = 0.0f;
					g = 0.0f;
					b = 0.5f;
				}
				
				// draw each line
				this.drawLine(gl, from.x, from.y, from.z, to.x, to.y, to.z, r, g, b);
			}
		}
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawTravelPath(GL gl)
	{
		// travel jump
		float r, g, b;
		r = 1.0f;
		g = 0.5f;
		b = 0.0f;
		
		// for each vector in this vector
		for (Iterator i = this.travelJumps.iterator(); i.hasNext();)
		{
			// each vector holds solarsystems
			Vector vec = (Vector) i.next();
			
			SolarSystem prev = null;
			SolarSystem next = null;
			for (Iterator y = vec.iterator(); y.hasNext();)
			{
				if (prev == null)
				{
					prev = (SolarSystem) y.next();
				}
				
				// get next system
				next = (SolarSystem) y.next();
				
				// draw line
				this.drawLine(gl, prev.x, prev.y, prev.z, next.x, next.y, next.z, r, g, b);
				
				// next becomes prev
				prev = next;
			}
		}
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawFoundSystem(GL gl, SolarSystem sys)
	{
		float r, g, b;
		r = 1.0f;
		g = 0.5f;
		b = 0.0f;
		
		this.drawLine(gl, sys.x - 100000, sys.y, sys.z, sys.x + 100000, sys.y, sys.z, r, g, b);
		// this.drawLine(gl, sys.x, sys.y - 100000, sys.z, sys.x, sys.y + 100000, sys.z, r, g, b);
		this.drawLine(gl, sys.x, sys.y, sys.z - 100000, sys.x, sys.y, sys.z + 100000, r, g, b);
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawRegions(GL gl)
	{
		Enumeration num = this.regions.elements();
		
		// loop through all regions
		while(num.hasMoreElements())
		{
			// get the region
			Region region = (Region) num.nextElement();
			
			if (region != null)
			{	
				// only draw outlines when enabled
				if (this.drawOutlines == true)
				{
					// region has to be within a certain angle
					if (KwCalc.calcDistance(-region.x, -region.z, (region.y * this.heightScale), this.viewX, this.viewY, -this.viewZ) < this.REGIONRANGE)
					{
						double angle = KwCalc.calcViewAngle(this.viewX, this.viewY, -this.viewZ, -region.x, -region.z, region.y);
						if (angle < 25 && angle > -25)
						{
							this.drawConvex(gl, region);
							
							// only draw region name when outlined
							// draw the text - make sure its always on top
							gl.glDisable(GL.GL_DEPTH_TEST);
							gl.glColor3f(0.5f, 0.5f, 0.5f);
							gl.glRasterPos3d((region.x), (region.z), (region.y * this.heightScale));
							this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_12, region.name);
							gl.glEnable(GL.GL_DEPTH_TEST);
						}
					}
				}
				else
				{
					// draw the text - make sure its always on top
					gl.glDisable(GL.GL_DEPTH_TEST);
					gl.glColor3f(0.5f, 0.5f, 0.5f);
					gl.glRasterPos3d((region.x), (region.z), (region.y * this.heightScale));
					this.glut.glutBitmapString(GLUT.BITMAP_HELVETICA_12, region.name);
					gl.glEnable(GL.GL_DEPTH_TEST);
				}
			}
		}
	}
	
	/**
	 * 
	 *
	 */
	private void setHeightScale()
	{
		// TODO: make it timebound, not fps bound
		// in 3d mode, going to flat
		if (this.drawFlat == true && this.heightScale > 0)
		{
			this.heightScale -= 0.01;
			if (this.heightScale < 0)
			{
				this.heightScale = 0;
			}
		}
		// in flat mode, going to 3d mode
		else if (this.drawFlat == false && this.heightScale < 1)
		{
			this.heightScale += 0.01;
			if (this.heightScale > 1)
			{
				this.heightScale = 1;
			}
		}
	}
	
	/**
	 * 
	 * @param start
	 * @param end
	 */
	private Vector getTravelPath(String start, String end)
	{
		// empty solarsystems
		SolarSystem sysStart, sysEnd;
		sysStart = null;
		sysEnd = null;
		Vector result = null;
		
		// loop through all solarsystems
		Enumeration num = this.solarSystems.elements();
		while(num.hasMoreElements())
		{
			SolarSystem current = (SolarSystem) num.nextElement();
			
			// is this solar system either our start or endpoint?
			if (current.name.equalsIgnoreCase(start))
			{
				sysStart = current;
			}
			else if (current.name.equalsIgnoreCase(end))
			{
				sysEnd = current;
			}
		}
		
		// both should not be null
		if (sysStart != null && sysEnd != null)
		{
			KwCalc calculator = new KwCalc();
			result = calculator.calcJumps(this.systemJumps, sysStart, sysEnd, this.avoidance);
		}
		
		return result;
	}
	
	/**
	 * 
	 * @param avoidance
	 */
	public void calcWaypoints(int avoidance)
	{
		// this will take all waypoints, and calculate the path between them
		this.avoidance = avoidance;
		this.travelJumps.clear();
		
		// only if there's enough systems
		if (this.travelSystems.size() >= 2)
		{
			String prev = null;
			String next = null;
			for (Iterator i = this.travelSystems.iterator(); i.hasNext();)
			{
				if (prev == null)
				{
					prev = (String) i.next();
				}
				
				// get the next system
				next = (String) i.next();
				
				// Calculate the waypoint
				Vector path = this.getTravelPath(next, prev);
				if (path != null)
				{
					this.travelJumps.add(path);
				}
				
				// next becomes previous
				prev = next;
			}
		}
	}
	
	/**
	 * 
	 * @param gl
	 */
	private void drawConvex(GL gl, Region curRegion)
	{
		// we now have all the points of this region
		Vector regHull = (Vector) this.convexSystems.get(curRegion);
		
		ConvexPoint previous = null;
		
		Enumeration num = regHull.elements();
		while (num.hasMoreElements())
		{
			ConvexPoint p = (ConvexPoint) num.nextElement();
			
			if (previous == null)
			{
				previous = p;
				p = (ConvexPoint) num.nextElement();
			}
			
			// draw a line
			this.drawLine(gl, previous.x, curRegion.minZ, previous.z, p.x, curRegion.minZ, p.z,curRegion.color[0], curRegion.color[1], curRegion.color[2]);
			this.drawLine(gl, previous.x, curRegion.maxZ, previous.z, p.x, curRegion.maxZ, p.z,curRegion.color[0], curRegion.color[1], curRegion.color[2]);
			
			// draw the quad
			if (this.heightScale == 1)
			{
				gl.glColor4f(curRegion.color[0], curRegion.color[1], curRegion.color[2], 0.2f);
				gl.glBegin(GL.GL_QUADS);
				gl.glVertex3d(previous.x, previous.z, curRegion.minZ);
				gl.glVertex3d(p.x, p.z, curRegion.minZ);
				gl.glVertex3d(p.x, p.z, curRegion.maxZ);
				gl.glVertex3d(previous.x, previous.z, curRegion.maxZ);
				gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
				gl.glEnd();
			}
			previous = p;	
		}
		
		// close the gap. Connect last with first
		ConvexPoint p = (ConvexPoint) regHull.firstElement();
		ConvexPoint p2 = (ConvexPoint) regHull.lastElement();
		
		// draw lines
		this.drawLine(gl, p2.x, curRegion.minZ - 0.01f, p2.z, p.x, curRegion.minZ, p.z - 0.01f, curRegion.color[0], curRegion.color[1], curRegion.color[2]);
		this.drawLine(gl, p2.x, curRegion.maxZ - 0.01f, p2.z, p.x, curRegion.maxZ, p.z - 0.01f, curRegion.color[0], curRegion.color[1], curRegion.color[2]);
		
		if (this.heightScale == 1)
		{
			gl.glColor4f(curRegion.color[0], curRegion.color[1], curRegion.color[2], 0.2f);
			gl.glBegin(gl.GL_QUADS);
			gl.glVertex3d(p2.x, p2.z, curRegion.minZ);
			gl.glVertex3d(p.x, p.z, curRegion.minZ);
			gl.glVertex3d(p.x, p.z, curRegion.maxZ);
			gl.glVertex3d(p2.x, p2.z, curRegion.maxZ);
			gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
			gl.glEnd();
		}
	}
	
	public void toggleFlat()
	{
		this.drawFlat = !this.drawFlat;
	}
	
	public void toggleOutline()
	{
		this.drawOutlines = !this.drawOutlines;
	}
	
	public void toggleSystem()
	{
		this.drawSystemNames = !this.drawSystemNames;
	}
	
	public void toggleTactical()
	{
		this.tactical = !this.tactical;
	}
	
	public void findSystem(String sys)
	{
		this.searchedSystem = sys;
	}
	
	public DBReader getDBReader()
	{
		return this.dbReader;
	}
	
	public void toggleSecStat()
	{
		this.showSecStat = !this.showSecStat;
	}
	

	public void mouseClicked(MouseEvent arg0) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}

	public void mousePressed(MouseEvent e) 
	{
		// increase buttons pressed
		this.btnsPressed += 1;
		
		// only left OR right button
		if (e.getButton() == 1 || e.getButton() == 3 && this.btnsPressed == 1)
		{
			// if the startx and y are -1, it means its not set
			if (this.mStartX == -1 && this.mStartY == -1)
			{
				this.mStartX = e.getX();
				this.mStartY = e.getY();
			}
		}
		// 2 or more buttons pressed
		else if (this.btnsPressed >= 2)
		{
			// set the startx and y
			this.mStartX = e.getX();
			this.mStartY = e.getY();
		}
	}

	public void mouseReleased(MouseEvent e) 
	{
		// reset startx and y to -1
		this.mStartX = -1;
		this.mStartY = -1;
		
		// decrease buttons pressed
		this.btnsPressed -= 1;
	}

	public void mouseDragged(MouseEvent e) 
	{
		// if the startX and Y are set, mouse is pressed
		if (this.mStartX != -1 && this.mStartY != -1 && this.btnsPressed == 1)
		{
			int newx, newy;
			newx = e.getX();
			newy = e.getY();
			
			// calculate how much to move
			this.mMoveX = newx - this.mStartX;
			this.mMoveY = newy - this.mStartY;
			
			// set the new mStartX and mStartY
			this.mStartX = newx;
			this.mStartY = newy;
		}
		// modify the scaling - aka zooming in
		else if (this.mStartX != -1 && this.mStartY != -1 && this.btnsPressed >= 2)
		{
			// get the x and y
			int newx, newy;
			newx = e.getX();
			newy = e.getY();
			
			// make new scale
			this.mMoveZ = newy - this.mStartY;
			
			// set the new mStartX and mStartY
			this.mStartX = newx;
			this.mStartY = newy;
		}
	}

	public void mouseMoved(MouseEvent e) 
	{
		if (e.getButton() == 0)
		{
			this.btnsPressed = 0;
		}
	}

	public void keyPressed(KeyEvent e) 
	{
		// button was pressed (duh)
		// System.out.println(e.getKeyCode());
		
		switch(e.getKeyCode())
		{
			// the f button
			// toggle flat or depth mode
			case 70:
				this.toggleFlat();
				break;
			
			// the o button
			// toggle draw outlines
			case 79:
				this.toggleOutline();
				break;
				
			// the t button
			// toggle tactical mode
			//case 84:
				//this.toggleTactical();
				//break;
				
			// the s button
			case 83:
				this.toggleSecStat();
				break;
				
			// the n button
			case 78:
				this.toggleSystem();
				break;
		}
	}

	public void keyReleased(KeyEvent e) {
	}

	public void keyTyped(KeyEvent e) {
	}

	public void mouseWheelMoved(MouseWheelEvent e) 
	{
		this.mMoveZ = e.getWheelRotation() * -30;
	}
}
